# The Algorithms - Go
[![Gitpod Ready-to-Code](https://img.shields.io/badge/Gitpod-Ready--to--Code-blue?logo=gitpod&style=flat-square)](https://gitpod.io/#https://github.com/TheAlgorithms/Go)&nbsp;
![golangci-lint](https://github.com/TheAlgorithms/Go/workflows/golangci-lint/badge.svg)
![](https://img.shields.io/github/repo-size/TheAlgorithms/Go.svg?label=Repo%20size&style=flat-square)&nbsp;
![update_directory_md](https://github.com/TheAlgorithms/Go/workflows/update_directory_md/badge.svg)
[![Discord chat](https://img.shields.io/discord/808045925556682782.svg?logo=discord&colorB=7289DA&style=flat-square)](https://discord.gg/c7MnfGFGa6)&nbsp;

### Algorithms implemented in Go (for education)

The repository is a collection of open-source implementation of a variety of algorithms implemented in Go and licensed under [MIT License](LICENSE).

Read our [Contribution Guidelines](CONTRIBUTING.md) before you contribute.

## List of Algorithms


## Ciphers
* [Caesar](./ciphers/caesar/)
* [Diffie Hellman Key Exchange](./ciphers/diffie_hellman_key_exchange/)
* [Polybius](./ciphers/polybius/)
* [Rot13](./ciphers/rot13/)
* [Rsa](./ciphers/rsa/)
* [Xor](./ciphers/xor/)

## Conversions
* [Roman To Integer](./conversions/roman_to_integer/)

## Data Structures
* [Binary Tree](./data_structures/binary_tree/)
* [Dynamic Array](./data_structures/dynamic_array/)
* [Hashmap](./data_structures/hashmap/)
* Linked-List
    * [Doubly Linked-list](./data_structures/linkedlist/doubly_linkedlist/)
    * [Singly Linked-list](./data_structures/linkedlist/singly_linkedlist/)
* [Queue](./data_structures/queue/) | Array, Linked-List, STL(Container/List), Test
* [Stack](./data_structures/stack/) | Array, Linked-List, STL(Container/List), Test
* [Set](./data_structures/set/)
* [Trie](./data_structures/trie/)

## Design Patterns
* [Abstract Factory](./design_patterns/abstract_factory/)
* [Builder](./design_patterns/builder/)
* [Factory Method](./design_patterns/factory_method/)
* [Prototype](./design_patterns/prototype/)


## Dynamic Programming
* [Binomial Coefficient](./dynamic_programming/binomialcoefficient.go)
* [Fibonacci](./dynamic_programming/fibonacci.go) | [Test](./dynamic_programming/fibonacci_test.go)
* [Knapsack](./dynamic_programming/knapsack.go)
* [Longest Common Subsequence](./dynamic_programming/longestcommonsubsequence.go)
* [Longest Palindromic Subsequence](./dynamic_programming/longestpalindromicsubsequence.go)
* [Matrix Multiplication](./dynamic_programming/matrixmultiplication.go)
* [Cutting a Rod](./dynamic_programming/rodcutting.go)

## Genetic Algorithm
* [Genetic Algorithm](./genetic_algorithm/geneticalgorithm.go)

## Graphs
* [Breadth First Search](./graphs/breadth_first_search/)
* [Depth First Search](./graphs/depth_first_search/)
* [Floyd Warshall](./graphs/floyd_warshall/)
    
## Math
* [Gcd](./math/gcd/)
* [Lcm](./math/lcm/)
* [Modular](./math/modular/) 
* [Permutation](./math/permutation/)
* [Power](./math/power/)
* [Prime](./math/prime/)
* [Pythagoras](./math/pythagoras/)
* [Sieve](./math/sieve/)


## Other
* [Max Subarray Sum](./other/max_subarray_sum/)
* [Monte Carlo Pi](./other/monte_carlo_pi/)
* [Nested Brackets](./other/nested_brackets/)
* [Password Generator](./other/password_generator/)
* [String Combinations](./other/string_combinations/)

## Searches
* [Binary Search](./searches/binary_search/)
* [Linear Search](./searches/linear_search/)

## Sorts
* [Bubble Sort](./sorts/bubblesort.go)
* [Heap Sort](./sorts/heapsort.go)
* [Insertion Sort](./sorts/insertionsort.go)
* [Merge Sort](./sorts/mergesort.go)
* [Quick Sort](./sorts/quicksort.go)
* [Radix Sort](./sorts/radixsort.go)
* [Selection Sort](./sorts/selectionsort.go)
* [Shell Sort](./sorts/shellsort.go)


* Sorts Test
  * [Sorts Test](./sorts/sorts_test.go)
  * [Sorts Testcases](./sorts/sorts_testcases.go)

## Strings
* [Levenshtein Distance](./strings/levenshtein_distance/)
* Multiple String Matching
    * [Advanced Aho Corasick](./strings/multiple_string_matching/advanced_aho_corasick/)
    * [Aho Corasick](./strings/multiple_string_matching/aho_corasick/)
* [Naive String Search](./strings/naive_string_search/)
* Single String Matching
    * [Bom](./strings/single_string_matching/bom/)
    * [Horspool](./strings/single_string_matching/horspool/)
    * [Kmp](./strings/single_string_matching/kmp/)
